@@ -2,7 +2,7 @@
 #this repository contains the full copyright notices and license terms.
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.pyson import Eval
-from trytond.backend import TableHandler
+from trytond import backend
 from trytond.transaction import Transaction
 from trytond.pool import Pool
 
@@ -23,7 +23,7 @@ class Work(ModelSQL, ModelView):
             ],
         'Type', required=True, select=True)
     company = fields.Function(fields.Many2One('company.company', 'Company'),
-        'get_company', searcher='search_comany')
+        'on_change_with_company', searcher='search_comany')
     party = fields.Many2One('party.party', 'Party',
         states={
             'invisible': Eval('type') != 'project',
@@ -34,13 +34,10 @@ class Work(ModelSQL, ModelView):
             'invisible': Eval('type') != 'project',
             }, depends=['party', 'type'])
     timesheet_available = fields.Function(
-        fields.Boolean('Available on timesheets'), 'get_timesheet_available')
+        fields.Boolean('Available on timesheets'),
+        'on_change_with_timesheet_available')
     hours = fields.Function(fields.Float('Timesheet Hours', digits=(16, 2),
-            states={
-                'invisible': ~Eval('timesheet_available'),
-                },
-            depends=['timesheet_available'],
-            help="Total time spent on this work"), 'get_hours')
+            help="Total time spent on this work"), 'on_change_with_hours')
     effort = fields.Float("Effort",
         states={
             'invisible': Eval('type') != 'task',
@@ -56,9 +53,12 @@ class Work(ModelSQL, ModelView):
             ('opened', 'Opened'),
             ('done', 'Done'),
             ], 'State', required=True, select=True)
-    sequence = fields.Integer('Sequence',
-        order_field='(%(table)s.sequence IS NULL) %(order)s, '
-        '%(table)s.sequence %(order)s')
+    sequence = fields.Integer('Sequence')
+
+    @staticmethod
+    def order_sequence(tables):
+        table, _ = tables[None]
+        return [table.sequence == None, table.sequence]
 
     @staticmethod
     def default_type():
@@ -75,6 +75,7 @@ class Work(ModelSQL, ModelView):
     @classmethod
     def __register__(cls, module_name):
         TimesheetWork = Pool().get('timesheet.work')
+        TableHandler = backend.get('TableHandler')
         cursor = Transaction().cursor
         table_project_work = TableHandler(cursor, cls, module_name)
         table_timesheet_work = TableHandler(cursor, TimesheetWork, module_name)
@@ -85,16 +86,16 @@ class Work(ModelSQL, ModelView):
 
         # Migration from 2.0: copy sequence from timesheet to project
         if migrate_sequence:
-            cursor.execute(
-                'SELECT t.sequence, t.id '
-                'FROM "%s" AS t '
-                'JOIN "%s" AS p ON (p.work = t.id)' % (
-                    TimesheetWork._table, cls._table))
+            project = cls.__table__()
+            timesheet = TimesheetWork.__table__()
+            cursor.execute(*timesheet.join(project,
+                    condition=project.work == timesheet.id
+                    ).select(timesheet.sequence, timesheet.id))
             for sequence, id_ in cursor.fetchall():
-                sql = ('UPDATE "%s" '
-                    'SET sequence = %%s '
-                    'WHERE work = %%s' % cls._table)
-                cursor.execute(sql, (sequence, id_))
+                cursor.execute(*project.update(
+                        columns=[project.sequence],
+                        values=[sequence],
+                        where=project.work == id_))
 
         # Migration from 2.4: drop required on sequence
         table_project_work.not_null_action('sequence', action='remove')
@@ -109,10 +110,10 @@ class Work(ModelSQL, ModelView):
         cls._order.insert(0, ('sequence', 'ASC'))
         cls._error_messages.update({
                 'invalid_parent_state': ('Work "%(child)s" can not be opened '
-                    'because its parent work "%(parent)s" is already done."'),
+                    'because its parent work "%(parent)s" is already done.'),
                 'invalid_children_state': ('Work "%(parent)s" can not be '
                     'done because its child work "%(child)s" is still '
-                    'opened."'),
+                    'opened.'),
                 })
 
     @classmethod
@@ -147,7 +148,7 @@ class Work(ModelSQL, ModelView):
         return self.work.active
 
     @classmethod
-    def set_active(self, works, name, value):
+    def set_active(cls, works, name, value):
         pool = Pool()
         Work = pool.get('timesheet.work')
 
@@ -159,18 +160,21 @@ class Work(ModelSQL, ModelView):
     def search_active(cls, name, clause):
         return [('work.active',) + tuple(clause[1:])]
 
-    def get_company(self, name):
-        return self.work.company.id
+    @fields.depends('work')
+    def on_change_with_company(self, name=None):
+        return self.work.company.id if self.work else None
 
     @classmethod
     def search_comany(cls, name, clause):
         return [('work.company',) + tuple(clause[1:])]
 
-    def get_timesheet_available(self, name):
-        return self.work.timesheet_available
+    @fields.depends('work')
+    def on_change_with_timesheet_available(self, name=None):
+        return self.work.timesheet_available if self.work else None
 
-    def get_hours(self, name):
-        return self.work.hours
+    @fields.depends('work')
+    def on_change_with_hours(self, name=None):
+        return self.work.hours if self.work else None
 
     @classmethod
     def get_parent(cls, project_works, name):
@@ -267,33 +271,35 @@ class Work(ModelSQL, ModelView):
 
         return [('work', 'in', tw_ids)]
 
-    @staticmethod
-    def sum_tree(works, getter):
-        leafs = set()
+    @classmethod
+    def sum_tree(cls, works, getter):
         result = {}
+        parents = {}
         for work in works:
-            if not work.children:
-                leafs.add(work)
             result[work.id] = getter(work)
-
-        works = set(works)
+            parent = work.parent
+            if parent:
+                parents[work.id] = parent.id
+        works = set((w.id for w in works))
+        leafs = works - set(parents.itervalues())
         while leafs:
             for work in leafs:
                 works.remove(work)
-                if work.parent and work.parent.id in result:
-                    result[work.parent.id] += result[work.id]
-            next_leafs = set(w for w in works)
+                parent = parents.get(work)
+                if parent in result:
+                    result[parent] += result[work]
+            next_leafs = set(works)
             for work in works:
-                if not work.parent:
+                parent = parents.get(work)
+                if not parent:
                     continue
-                if work.parent.id in next_leafs and work.parent in works:
-                    next_leafs.remove(work.parent)
+                if parent in next_leafs and parent in works:
+                    next_leafs.remove(parent)
             leafs = next_leafs
         return result
 
     @classmethod
     def get_total_effort(cls, works, name):
-
         works = cls.search([
                 ('parent', 'child_of', [w.id for w in works]),
                 ('active', '=', True),
@@ -311,11 +317,13 @@ class Work(ModelSQL, ModelView):
         for key in timesheet_default.keys():
             if key in cls._fields:
                 del timesheet_default[key]
+        timesheet_default['children'] = None
         new_project_works = []
         for project_work in project_works:
             timesheet_work, = TimesheetWork.copy([project_work.work],
                 default=timesheet_default)
             pwdefault = default.copy()
+            pwdefault['children'] = None
             pwdefault['work'] = timesheet_work.id
             new_project_works.extend(super(Work, cls).copy([project_work],
                     default=pwdefault))
@@ -331,3 +339,9 @@ class Work(ModelSQL, ModelView):
         super(Work, cls).delete(project_works)
 
         TimesheetWork.delete(timesheet_works)
+
+    @classmethod
+    def search_global(cls, text):
+        for id_, rec_name, icon in super(Work, cls).search_global(text):
+            icon = icon or 'tryton-project'
+            yield id_, rec_name, icon
